import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class ListAppBar<T extends IdItemInterface> extends StatefulWidget
    implements PreferredSizeWidget {
  final Widget? title;
  final Widget? selectionTitle;
  final bool showCountInDescription;
  final Widget? description;

  final bool? showBackArrow;
  final bool showSelectAllButton;

  final List<Widget?>? appBarButtons;
  final List<Widget?>? appBarPopupButtons;

  const ListAppBar({
    Key? key,
    this.title,
    this.showBackArrow,
    this.showSelectAllButton = false,
    this.appBarButtons,
    this.appBarPopupButtons,
    this.showCountInDescription = false,
    this.selectionTitle,
    this.description,
  }) : super(key: key);

  @override
  State<ListAppBar<T>> createState() => _ListAppBarState<T>();

  @override
  Size get preferredSize => AppBar(
        title: Text("Test"),
      ).preferredSize;
}

class _ListAppBarState<T extends IdItemInterface> extends State<ListAppBar<T>> {
  FocusNode searchFocusNode = FocusNode();

  void listener() {
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    ListController.of(context).state.removeListener(listener);
    ListController.of(context).state.addListener(listener);

    return AnimatedSwitcher(
      duration: Duration(milliseconds: 300),
      child: ListController.of(context).state.searchIsActive
          ? buildSearchAppBar(context)
          : (ListController.of(context).state.selectionIsActive
              ? buildSelectionAppBar(context)
              : buildDefaultAppBar(context)),
    );
  }

  Widget buildSearchAppBar(BuildContext context) {
    Future.delayed(Duration.zero, () => searchFocusNode.requestFocus());
    return DefaultAppBar(
      key: ValueKey("search"),
      showBackArrow: false,
      titleFullSpace: true,
      title: SearchBar(
        controller: ListController.of(context).state.searchController,
        focusNode: searchFocusNode,
      ),
    );
  }

  Widget buildSelectionAppBar(BuildContext context) {
    var length = ListController.of(context).state.selectedRefs.length;
    return DefaultAppBar(
      key: ValueKey("selection"),
      title: widget.selectionTitle ?? Text("${length.toString()} ausgewählt"),
      showBackArrow: true,
      appBarButtons: [buildSearchButton(context), getSelectionAppbarButton()],
      appBarPopupButtons: getAppBarPopupButtons(context),
    );
  }

  Widget buildDefaultAppBar(BuildContext context) {
    return DefaultAppBar(
      key: ValueKey("default"),
      title: widget.title,
      description: widget.description ??
          (ListController.of(context).state.allReferences != null &&
                  widget.showCountInDescription
              ? Text(ListController.of(context)
                      .state
                      .allReferences!
                      .length
                      .toString() +
                  " angezeigt")
              : null),
      showBackArrow: widget.showBackArrow,
      appBarButtons: [
        buildSearchButton(context),
        ...(widget.appBarButtons ?? [])
      ],
      appBarPopupButtons: getAppBarPopupButtons(context),
    );
  }

  List<Widget?> getAppBarPopupButtons(BuildContext context) {
    var list = <Widget?>[];

    if (widget.appBarPopupButtons != null)
      list.addAll(widget.appBarPopupButtons!);

    if (widget.showSelectAllButton &&
        !ListController.of(context).state.selectionIsActive &&
        ListController.of(context).state.multiSelect)
      list.add(buildSelectAllButton(context));

    if (ListController.of(context).state.selectionIsActive &&
        widget.appBarButtons != null) list.addAll(widget.appBarButtons!);

    return list;
  }

  Widget buildSelectAllButton(BuildContext context) {
    return IconButton(
      icon: Icon(Icons.check_circle),
      onPressed: () {
        ListController.of(context).state.selectAll();
      },
      tooltip: "Alles auswählen",
    );
  }

  Widget? buildSearchButton(BuildContext context) {
    return IconButton(
      icon: Icon(
        Icons.search,
      ),
      onPressed: () {
        ListController.of(context).state.searchIsActive = true;
      },
      tooltip: "Suchen",
    );
  }

  Widget? getSelectionAppbarButton() {
    if (!ListController.of(context).state.multiSelect) return null;

    var all = ListController.of(context).state.selectedRefs.length ==
        (ListController.of(context).state.allReferences?.length ?? -1);
    var none = ListController.of(context).state.selectedRefs.length == 0;

    return IconButton(
      tooltip: all ? "Auswahl verwerfen" : "Alle auswählen",
      onPressed: () => setState(() {
        if (all) {
          ListController.of(context).state.selectedRefs.clear();
          ListController.of(context).state.notifyListeners();
        } else {
          ListController.of(context).state.selectAll();
        }
      }),
      icon: Icon(
        all
            ? Icons.check_circle
            : (none
                ? Icons.radio_button_unchecked
                : Icons.check_circle_outline),
        color: all ? DefaultColors.accent : DefaultColors.text,
      ),
    );
  }
}
