import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class BasicScaffold extends StatefulWidget {
  final Widget Function(BuildContext context) builder;
  final PreferredSizeWidget? Function(BuildContext context)? appBarBuilder;
  final Widget? Function(BuildContext context)? menuBarBuilder;
  final Widget? Function(BuildContext context)? drawerBuilder;

  final Future<int>? Function()? preparationFuture;
  final Widget Function(BuildContext context)? builderPreparation;
  final PreferredSizeWidget? Function(BuildContext context)?
      appBarBuilderPreparation;
  final Widget? Function(BuildContext context)? menuBarBuilderPreparation;
  final Widget? Function(BuildContext context)? drawerBuilderPreparation;

  final Future<bool> Function(BuildContext context)? onWillPop;
  final bool scrollable;

  final IHelpWidget? helpWidget;

  const BasicScaffold({
    Key? key,
    required this.builder,
    this.appBarBuilder,
    this.menuBarBuilder,
    this.preparationFuture,
    this.builderPreparation,
    this.appBarBuilderPreparation,
    this.menuBarBuilderPreparation,
    this.onWillPop,
    this.scrollable = true,
    this.helpWidget,
    this.drawerBuilder,
    this.drawerBuilderPreparation,
  }) : super(key: key);

  @override
  State<BasicScaffold> createState() => _BasicScaffoldState();
}

class _BasicScaffoldState extends State<BasicScaffold> {
  GlobalKey<ScaffoldState> scaffoldKey = GlobalKey<ScaffoldState>();
  ScaffoldPageControllerContent scaffoldPageControllerContent =
      ScaffoldPageControllerContent();

  @override
  Widget build(BuildContext context) {
    var preparing = isPreparing();

    return ScaffoldPageController(
      content: scaffoldPageControllerContent,
      child: Builder(
          builder: (context) => WillPopScope(
                onWillPop: () => onWillPop(context),
                child: Shimmer(
                  child: Scaffold(
                    key: scaffoldKey,
                    drawer: buildDrawer(context, preparing),
                    appBar: buildAppBar(context, preparing),
                    body: buildBodyContainer(buildBody(context, preparing)),
                    bottomNavigationBar: buildMenuBar(context, preparing),
                  ),
                ),
              )),
    );
  }

  Future<bool> onWillPop(BuildContext context) {
    return widget.onWillPop != null
        ? widget.onWillPop!.call(context)
        : Future.value(true);
  }

  Widget buildBodyContainer(Widget body) {
    return Stack(
      children: <Widget>[
        Container(
          decoration: BoxDecoration(
            color: DefaultColors.background,
          ),
        ),
        SafeArea(
          child: widget.scrollable
              ? SingleChildScrollView(
                  child: body,
                )
              : body,
        ),
        widget.helpWidget != null
            ? HelpSideWidget(helpWidget: widget.helpWidget!)
            : Container(),
      ],
    );
  }

  Widget? buildDrawer(BuildContext context, bool preparing) {
    if (preparing && widget.drawerBuilderPreparation != null) {
      return widget.drawerBuilderPreparation?.call(context);
    } else {
      return widget.drawerBuilder?.call(context);
    }
  }

  Widget? buildMenuBar(BuildContext context, bool preparing) {
    if (preparing && widget.menuBarBuilderPreparation != null) {
      return widget.menuBarBuilderPreparation!.call(context);
    } else {
      return widget.menuBarBuilder?.call(context);
    }
  }

  PreferredSizeWidget? buildAppBar(BuildContext context, bool preparing) {
    if (preparing && widget.appBarBuilderPreparation != null) {
      return widget.appBarBuilderPreparation!.call(context);
    } else {
      return widget.appBarBuilder?.call(context);
    }
  }

  Widget buildBody(BuildContext context, bool preparing) {
    if (preparing) {
      return widget.builderPreparation?.call(context) ?? ScaffoldShimmer();
    } else {
      return widget.builder.call(context);
    }
  }

  bool isPreparing() {
    var future = widget.preparationFuture?.call();
    if (future == null) {
      return false;
    } else {
      future.whenComplete(() {
        if (mounted) setState(() {});
      });
      return true;
    }
  }
}
